searchData={"items":[{"type":"module","title":"EctoModel","doc":"# EctoModel\n\nEctoModel is a library that aims to overhaul your `Ecto.Schema`s with additional functionality such as easy, fluent querying of data as well as easy soft deletes!","ref":"EctoModel.html"},{"type":"module","title":"Installation - EctoModel","doc":"This package can be installed by adding `ecto_model` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_model, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"EctoModel.html#module-installation"},{"type":"module","title":"Features - EctoModel","doc":"Currently, EctoModel provides the following features:\n\n- Easy and fluent API for querying your data with `EctoModel.Queryable`\n- Easy and compile-time validated soft delete functionality with `EctoModel.SoftDelete`\n\nAll of the provided functionality is provided in an opt-in basis, so you can mix and match the functionality\nyou need as needed.\n\nPlease see the documentation for each feature for more information.","ref":"EctoModel.html#module-features"},{"type":"module","title":"License - EctoModel","doc":"EctoModel is released under the MIT License.","ref":"EctoModel.html#module-license"},{"type":"module","title":"Links - EctoModel","doc":"EctoModel is built on top of the following libraries, and as such, you may care about their documentation as well:\n\n- [Ecto](https://hexdocs.pm/ecto/Ecto.html)\n- [EctoMiddleware](https://hexdocs.pm/ecto_middleware/EctoMiddleware.html)\n- [EctoHooks](https://hexdocs.pm/ecto_hooks/EctoHooks.html)","ref":"EctoModel.html#module-links"},{"type":"module","title":"Contributing - EctoModel","doc":"We don't currently have any contributing guidelines, but if you'd like to contribute, please feel free to open an issue or a pull request.\n\nPlease note that we do enforce 100% test coverage, so any changes will need to be accompanied by tests.\n\nAdditionally, we withhold the right to refuse any changes that we feel do not align with the goals of the project.","ref":"EctoModel.html#module-contributing"},{"type":"behaviour","title":"EctoModel.Queryable","doc":"A behaviour for defining a `query/2` callback that can be used as an easy-to-use and fluent DSL for building\nEcto queries in a consistent manner across different schemas.","ref":"EctoModel.Queryable.html"},{"type":"behaviour","title":"Usage - EctoModel.Queryable","doc":"1. Define a schema module and use the `EctoModel.Queryable` behaviour. By default, this is all you have to do\n   unless you wish to customize the behaviour further.\n\n   ```elixir\n   defmodule MyApp.User do\n     use Ecto.Schema\n     use EctoModel.Queryable\n\n     schema \"users\" do\n       field(:name, :string)\n       field(:email, :string)\n       field(:inserted_at, :utc_datetime)\n       field(:updated_at, :utc_datetime)\n     end\n   end\n\n   (iex)> MyApp.Repo.exists?(MyApp.User.query(name: \"John\", email: nil))\n   true\n   (iex)> MyApp.Repo.exists?(MyApp.User.query(email: nil, inserted_at: {:>=, ~U[2021-01-01 00:00:00Z]}))\n   false\n   ```\n\n2. Implement the `base_query/0` optional callback if you want all queries, by default, to inherit a standard set\n   of filters. This is useful for implementing soft deletes, for example.\n\n   ```elixir\n   def base_query, do: from(x in __MODULE__, where: is_nil(x.deleted_at))\n   ```\n\n3. Implement the `query/2` callback to apply filters to the query. This is where you can extend the default supported\n   filters or add custom filters that are specific to your schema.\n\n   When you use the `EctoModel.Queryable` behaviour, you get a default implementation of the `query/2` callback that\n   looks like the following:\n\n   ```elixir\n   def query(base_query \\ base_query(), filters) do\n     Enum.reduce(filters, base_query, &apply_filter(&2, &1))\n   end\n   ```\n\n   If you want to add new logic while still inheriting the default behaviour, you can do so by ensuring a clause exists\n   within your `Enum.reduce/3` implementation that matches against any pattern and delegates to the default behaviour\n   provided by `EctoModel.Queryable.apply_filter/2`.","ref":"EctoModel.Queryable.html#module-usage"},{"type":"behaviour","title":"Supported Filters - EctoModel.Queryable","doc":"By default, all non-embedded fields should be supported by the default `apply_filter/2` implementation for the following\noperators:\n\n- Equality (`==`) via `field: value`\n- Inclusion (`in`) via `field: [value1, value2]`\n- Exclusion (`not in`) via `field: {:not, [value1, value2]}`\n- Greater than (`>`) via `field: {:gt, value} or field: {:>, value}`\n- Greater than or equal to (`>=`) via `field: {:gte, value} or field: {:>=, value}`\n- Less than (`<`) via `field: {:lt, value} or field: {:<, value}`\n- Less than or equal to (`<=`) via `field: {:lte, value} or field: {:<=, value}`\n- Is null (`nil`) via `field: nil`\n- Is not null (`not nil`) via `field: {:not, nil}`\n- Like (`like %value&`) via `field: ~r/value/`\n- Case insensitive Like (`ilike %value&`) via `field: ~r/value/i`\n\nAdditionally, while not filters in the traditional sense, the following options are also supported:\n\n- Preloading (`preload`) via `preload: :association or preload: [:association1, :association2]`\n- Limiting (`limit`) via `limit: 10`\n- Offsetting (`offset`) via `offset: 10`\n- Ordering (`order_by`) via `order_by: :field or order_by: {:desc, :field} or order_by: [:field1, :field2]`","ref":"EctoModel.Queryable.html#module-supported-filters"},{"type":"function","title":"EctoModel.Queryable.apply_filter/2","doc":"List of default filters that can be used in a schema's `query/2` callback","ref":"EctoModel.Queryable.html#apply_filter/2"},{"type":"callback","title":"EctoModel.Queryable.base_query/0","doc":"","ref":"EctoModel.Queryable.html#c:base_query/0"},{"type":"function","title":"EctoModel.Queryable.implemented_by?/1","doc":"Returns true a given module implements the `Queryable` behaviour","ref":"EctoModel.Queryable.html#implemented_by?/1"},{"type":"callback","title":"EctoModel.Queryable.query/2","doc":"","ref":"EctoModel.Queryable.html#c:query/2"},{"type":"module","title":"EctoModel.SoftDelete","doc":"Module responsible for allowing your schemas to opt into soft delete functionality.","ref":"EctoModel.SoftDelete.html"},{"type":"module","title":"Usage - EctoModel.SoftDelete","doc":"There are two things that need to happen in order to make a schema soft deletable:\n\n1) You need to ensure your `MyApp.Repo` module is using the `EctoMiddleware` behaviour, and you add the `EctoModel.SoftDelete` middleware to\n   the `middleware/2` callback before the `EctoMiddleware.Super` middleware.\n\n   This will enable `EctoModel.SoftDelete` to raise errors if users try to hard delete records when schemas have opted into soft deletes.\n\n   In future, we may add support for automatically delegating hard delete operations to transparently behave transparently as soft deletes in an\n   opt in basis.\n\n   You can also optionally add the following code to your `MyApp.Repo` module to enable easy soft delete operations:\n\n   ```elixir\n   def soft_delete!(resource, opts \\ []) do\n     EctoModel.SoftDelete.soft_delete!(resource, Keyword.put(opts, :repo, __MODULE__))\n   end\n\n   def soft_delete(resource, opts \\ []) do\n     EctoModel.SoftDelete.soft_delete(resource, Keyword.put(opts, :repo, __MODULE__))\n   end\n   ```\n\n2) You need to `use EctoModel.SoftDelete` in your schema, and configure the `field` and `type` options.\n\n   The specified field and type must match what is defined on said schema, though there are compile time validations provided for you to ensure\n   this remains in sync with your schema's natural evolution.\n\n   Additionally, if your schema also opts into implementing the `EctoModel.Queryable` behaviour, we automatically provide a `base_query/0`\n   implementation to will apply the neccessary filters to automatically filter out soft deleted records from query results.\n\n   If you need to specify a custom `base_query/0` implementation, you can do so while still inheriting the default behaviour provided when\n   using this module by calling `super()` in your custom implementation like so:\n\n   ```elixir\n   @impl EctoModel.Queryable\n   def base_query do\n     from x in ^super(), where: x.show_by_default != false\n   end\n   ```\n\nA full example of how to use `EctoModel.SoftDelete` is as follows:\n\n```elixir\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n  use EctoMiddleware\n\n  def middleware(_resource, _resolution) do\n    [EctoModel.SoftDelete, EctoMiddleware.Super]\n  end\nend\n\ndefmodule MyApp.User do\n  use Ecto.Schema\n  use EctoModel.SoftDelete, field: :deleted_at, type: :utc_datetime\n\n  schema \"users\" do\n    field(:name, :string)\n    field(:email, :string)\n    field(:deleted_at, :utc_datetime)\n  end\nend\n```","ref":"EctoModel.SoftDelete.html#module-usage"},{"type":"function","title":"EctoModel.SoftDelete.__after_compile__/2","doc":"After compile hook responsible for validating that a schema is properly configured for soft deletes.","ref":"EctoModel.SoftDelete.html#__after_compile__/2"},{"type":"macro","title":"EctoModel.SoftDelete.__using__/1","doc":"Persists the configuration for soft deletes on the schema, as well as providing a default impl. for `EctoModel.Queryable.base_query/0`.","ref":"EctoModel.SoftDelete.html#__using__/1"},{"type":"function","title":"EctoModel.SoftDelete.apply_filter!/2","doc":"Given a schema that has been configured to implement soft deletes, this function will apply the neccessary\nfilters to the query to ensure that soft deleted records are not included in the result set.\n\nNote that the strategy used for soft deletes is determined by the `type` option when `use`-ing `EctoModel.SoftDelete`,\nand we will apply the appropriate filter against the `field` option when `use`-ing `EctoModel.SoftDelete`.\n\nFor example, if a schema is configured to implement soft deletes like so:\n\n```elixir\ndefmodule MyApp.User do\n  use Ecto.Schema\n  use EctoModel.SoftDelete, field: :deleted_at, type: :utc_datetime\n\n  schema \"users\" do\n    field(:name, :string)\n    field(:email, :string)\n    field(:deleted_at, :utc_datetime)\n  end\nend\n```\n\nThen the `apply_filter!/2` function will apply the following filter to the query:\n\n```elixir\nfrom(x in query, where: is_nil(x.deleted_at))\n```\n\nHowever, if the schema is configured to implement soft deletes like so:\n\n```elixir\ndefmodule MyApp.User do\n  use Ecto.Schema\n  use EctoModel.SoftDelete, field: :deleted, type: :boolean\n\n  schema \"users\" do\n    field(:name, :string)\n    field(:email, :string)\n    field(:deleted, :boolean)\n  end\nend\n```\n\nThen the `apply_filter!/2` function will apply the following filter to the query:\n\n```elixir\nfrom(x in query, where: is_nil(x.deleted) or x.deleted == false)\n```","ref":"EctoModel.SoftDelete.html#apply_filter!/2"},{"type":"function","title":"EctoModel.SoftDelete.soft_delete/2","doc":"Will soft delete a given resource, and persist the changes to the database, based on that resource's configured\nsoft delete field and type.\n\nWill raise if given an entity that does not opt into soft deletes.","ref":"EctoModel.SoftDelete.html#soft_delete/2"},{"type":"function","title":"EctoModel.SoftDelete.soft_delete!/2","doc":"See `Ecto.Repo.soft_delete/2` for more information.","ref":"EctoModel.SoftDelete.html#soft_delete!/2"},{"type":"type","title":"EctoModel.SoftDelete.soft_delete_type/0","doc":"","ref":"EctoModel.SoftDelete.html#t:soft_delete_type/0"}],"content_type":"text/markdown"}